; Elektroniczna poziomica - obsuga akcelerometru MXD2125
; Rafa Baranowski (C) 2005

.INCLUDE "tn26def.inc"		; SEGMENT DEKLARACJI

.EQU SYS_FREQ	= 1		; czstotliwo systemowa w MHz
					; stae akcelerometru
.EQU K_AKCEL	= DDRA	;   rejestr kierunku portu
.EQU O_AKCEL	= PORTA	;   rejestr wyjciowy portu
.EQU I_AKCEL	= PINA	;   rejestr wejciowy portu
.EQU AKCEL_T	= 2		;   nr linii ADC dla sygnau temperatury
.EQU AKCEL_X	= 4		;   nr linii dla sygnau przypieszenia w osi X
.EQU AKCEL_Y	= 5		;   nr linii dla sygnau przypieszenia w osi Y
					; stae wywietlacza LCD
.EQU K_LCD		= DDRB	;   rejestr kierunku portu
.EQU O_LCD		= PORTB	;   rejestr wyjciowy portu
.EQU LCD_DB4	= 0		;   nr linii dla sygnau DB4
.EQU LCD_DB5	= 1		;   nr linii dla sygnau DB5
.EQU LCD_DB6	= 2		;   nr linii dla sygnau DB6
.EQU LCD_DB7	= 3		;   nr linii dla sygnau DB7
.EQU LCD_RS		= 4		;   nr linii dla sygnau RS
.EQU LCD_EN		= 5		;   nr linii dla sygnau EN


.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	rjmp	Reset			;   skok do procedury inicjalizujcej

					; deklaracje staych w pamici programu
T_powitanie:	.db	"Czujnik  MXD2125", ZNAK_KONCA
T_szablon1:		.db	"X:    ", 0xDF, "  Y:    ", 0xDF, ZNAK_KONCA
T_szablon2:		.db	"   T:     ", 0xDF, "C", ZNAK_KONCA


Reset:				; podprogram inicjalizujcy
	ldi	R16, RAMEND
	out	SP, R16		; inicjalizacja wskanika stosu

	rcall	Ini_akcel		; inicjalizacja akcelerometru

	rcall	Ini_LCD		; inicjalizacja wywietlacza LCD
	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_OFF)|(1<<LCD_SM_OFF)
	rcall	Instrukcja_LCD	; wyczanie kursora

	ldi	ZH, high(T_powitanie<<1)
	ldi	ZL, low(T_powitanie<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie powitania

	ldi	R16, 100
	ldi	R17, 10
	rcall	Czekaj_ms		; opnienie o 1 s

	rcall	Czysc_LCD		; czyszczenie wywietlacza

	ldi	R16, 0x00|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przejcie do grnego wiersza
	ldi	ZH, high(T_szablon1<<1)
	ldi	ZL, low(T_szablon1<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie szablonu grnego wiersza
	ldi	R16, 0x40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przejcie do dolnego wiersza
	ldi	ZH, high(T_szablon2<<1)
	ldi	ZL, low(T_szablon2<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie szablonu dolnego wiersza

Petla:				; ptla gwna programu
	ldi	R16, 1<<AKCEL_X
	rcall	Sprawdz_akcel	; zbadaj sygna przypieszenia w osi X
	rcall	Kat_akcel		; przelicz go na kt wzgldem Ziemi
					;   (warto kta znajduje si teraz w R17
					;   a znacznik T w SREG okrela jego znak)

	ldi	R16, (0x00+3)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesu kursor w miejsce wartoci X
	ldi	R16, 3
	rcall	Spacja_LCD		; zma poprzedni warto
	ldi	R16, (0x00+3)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesu kursor w miejsce wartoci X

	ldi	R16, '-'
	brts	Ptl_os_X
	ldi	R16, '+'
Ptl_os_X:
	rcall	Zapisz_LCD		; zalenie od znacznika T wywietl "+" lub "-"
	mov	R16, R17
	rcall	Pokaz_bajt_LCD	; wywietl warto kta w osi X


	ldi	R16, 1<<AKCEL_Y
	rcall	Sprawdz_akcel	; zbadaj sygna przypieszenia w osi Y
	rcall	Kat_akcel		; przelicz go na kt wzgldem Ziemi
					;   (warto kta znajduje si teraz w R17
					;   a znacznik T w SREG okrela jego znak)

	ldi	R16, (0x00+12)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesu kursor w miejsce wartoci Y
	ldi	R16, 3
	rcall	Spacja_LCD		; zma poprzedni warto
	ldi	R16, (0x00+12)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesu kursor w miejsce wartoci Y

	ldi	R16, '-'
	brts	Ptl_os_Y
	ldi	R16, '+'
Ptl_os_Y:
	rcall	Zapisz_LCD		; zalenie od znacznika T wywietl "+" lub "-"
	mov	R16, R17
	rcall	Pokaz_bajt_LCD	; wywietl warto kta w osi Y


	ldi	R16, (0x40+6)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesu kursor w miejsce wartoci T
	ldi	R16, 4
	rcall	Spacja_LCD		; zma poprzedni warto
	ldi	R16, (0x40+6)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przesu kursor w miejsce wartoci T

	in	R20, ADCL
	in	R21, ADCH		; odczytaj warto ostatnio pobranej prbki temp.
	subi	R20, low(int(1.125*1024/2.56))
	sbci	R21, high(int(1.125*1024/2.56))
					; odejmij od wyniku sta, odpowiadajc
					;   wartoci prbki dla temperatury zerowej
	mov	R17, R21
	mov	R16, R20
	lsr	R17			; dzielenie wartoci temperatury przez liczb 2
	ror	R16 			;   (po podzieleniu R16 zawiera cz cakowit
					;   wartoci temperatury w stopniach Celsjusza)

	rcall	Pokaz_bajt_LCD	; wywietl temperatur w stopniach Celsjusza

	ldi	R16, '.'
	rcall	Zapisz_LCD		; wywietl kropk

	ldi	R16, '0'		
	sbrc	R20, 0		; sprawd reszt z dzielenia temperatury przez 2
	ldi	R16, '5'
	rcall	Zapisz_LCD		; i wywietl j jako cz dziesitn temperatury

	ldi	R16, 100
	ldi	R17, 1
	rcall	Czekaj_ms		; wprowad opnienie 100 ms

	rjmp	Petla			; zaptl program


					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "akcel.inc"		;   obsuga akcelerometru
.INCLUDE "lcd.inc"		;   obsuga wywietlacza LCD
